﻿using System;
using MVR.FileManagementSecure;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.Events;
using SimpleJSON;

namespace ky1001
{
    public class PresetLoader : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "ky1001";
        public string pluginName = "PresetLoader";
        public string pluginVersion = "15.0";
        public string pluginDate = "2022-11-2";
        public string pluginDescription = "";
        #endregion

        private Atom _targetAtom;

        private const string _atomPersonPath = "Custom/Atom/Person/";
        private const string _pluginsPresetPath = "Custom/Atom/Person/Plugins";
        private const string _addonPackagesPath = "AddonPackages";
        private const string _filePrefs = "AddonPackagesFilePrefs/";
        private const string _pluginDataPath = "Saves/PluginData/ky1001/PresetLoader";
        private const string _clothing = "Clothing";
        private const string _hair = "Hair";
        private const string _pose = "Pose";
        private const string _skin = "Skin";
        private const string _appearance = "Appearance";
        private const string _plugins = "Plugins";
        private const string _clothingPresetsTempFile = "ky1001/PresetLoader_PluginPresets_tempfile";
        private const string _hairPresetsTempFile = "ky1001/PresetLoader_PluginPresets_tempfile";
        private const int _jsonstringLimit = 15000;
        private const float _currentPathHeight = 10f;
        private const float _presetPathHeight = 165f;

        private JSONStorableBool _enableRnd;
        private JSONStorableBool _enableAutoLoad;
        private JSONStorableBool _enableSubfolder;
        private JSONStorableBool _enablePositionAdjust;
        private Dictionary<string, JSONStorableBool> _enablePresetDic = new Dictionary<string, JSONStorableBool>();
        private Dictionary<string, JSONStorableBool> _onlyFavDic = new Dictionary<string, JSONStorableBool>();
        private Dictionary<string, JSONStorableString> _savedPathDic = new Dictionary<string, JSONStorableString>();
        private Dictionary<string, UIDynamicButton> _setPresetPathButton = new Dictionary<string, UIDynamicButton>();
        private Dictionary<string, UIDynamicToggle> _enablePresetToggle = new Dictionary<string, UIDynamicToggle>();
        private Dictionary<string, UIDynamicToggle> _onlyFavToggle = new Dictionary<string, UIDynamicToggle>();
        private Dictionary<string, UIDynamicTextField> _currentText = new Dictionary<string, UIDynamicTextField>();
        private Dictionary<string, UIDynamicTextField> _pathText = new Dictionary<string, UIDynamicTextField>();
        private Dictionary<string, string> _presetSystemPath = new Dictionary<string, string>();
        private Dictionary<string, JSONStorableString> _filesInFolder = new Dictionary<string, JSONStorableString>();
        private Dictionary<string, JSONStorableString> _current = new Dictionary<string, JSONStorableString>();
        private Dictionary<string, string> _presetPathDic = new Dictionary<string, string>();
        private Dictionary<string, JSONStorableString> _presetNumber = new Dictionary<string, JSONStorableString>();
        private JSONStorableStringChooser _selectCount;
        private UIDynamicButton _loadSettingButton;
        private UIDynamicButton _saveSettingButton;
        private UIDynamicButton _setPluginPresetButton;
        private UIDynamicButton _saveCurrentPresetsButton;
        private UIDynamicToggle _keepClothingToggle;
        private UIDynamicToggle _keepHairToggle;
        private UIDynamicToggle _enableDisappearToggle;
        private UIDynamicToggle _keepScaleToggle;
        private UIDynamicToggle _nudeToggle;
        private JSONStorableBool _isKeepClothing;
        private JSONStorableBool _isKeepHair;
        private JSONStorableBool _isEnableDisappear;
        private JSONStorableBool _isKeepScale;
        private JSONStorableBool _isNude;
        private UIDynamic _spLeft;
        private UIDynamic _spRight;
        private UIDynamic _spLeftBottom;
        private UIDynamic _spRightBottom;
        private Dictionary<string, string> _favPathDic = new Dictionary<string, string>();
        private Dictionary<string, List<string>> _listDic = new Dictionary<string, List<string>>();
        private Dictionary<string, List<string>> _presetListDic = new Dictionary<string, List<string>>();
        private Dictionary<string, List<string>> _favListDic = new Dictionary<string, List<string>>();
        private List<string> _presetList = new List<string>();
        private List<string> _favList = new List<string>();
        private List<string> _femaleList = new List<string>();
        private List<string> _activeClothingList = new List<string>();
        private string _presetName;
        private string _presetPath;
        private string _varFileName;
        private string _varFullName;
        private string _varFullPath;
        private Dictionary<string, int> _previousNumber = new Dictionary<string, int>();
        private string _previousSelectCount;
        private bool _isTopMenu = true;
        private bool _loopStart = false;
        private float _waitCount;
        private bool _isFemale;
        private string _appPreset;
        private bool _isRestart = false;
        private string _posePresetFile;
        private string _skinPresetFile;
        private string _loadData;
        private JSONNode _jSONNode;
        private JSONArray _asArray;
        private JSONClass _asObject;
        private int _startTime;
        private int _currentTime;
        private int _selectTime;
        private string _id;
        private Vector3 _personPosi;
        private Vector3 _loadedPosi;
        private bool _isPosition;
        private bool _isHip;
        private float _min;
        private Vector3 _control;
        private Vector3 _hipControl;
        private Vector3 _posi;
        private Vector3 _rota;
        private Dictionary<string, Vector3> _rigidbodyPosiLocal = new Dictionary<string, Vector3>();
        private Dictionary<string, Quaternion> _rigidbodyRotaLocal = new Dictionary<string, Quaternion>();
        private Dictionary<string, Vector3> _rigidbodyPosiWorld = new Dictionary<string, Vector3>();
        private Dictionary<string, Quaternion> _rigidbodyRotaWorld = new Dictionary<string, Quaternion>();
        private Vector3 _rootPosi;
        private Vector3 _rootRota;
        private int _poseWaitStart;
        private string _nextPreset = "Start";
        private Dictionary<string, List<int>> _randomOrder = new Dictionary<string, List<int>>();
        private Dictionary<string, UnityAction> _setPath = new Dictionary<string, UnityAction>();

        private string _presetTypePath;
        private string _saveName;
        private bool _isSavePresets = false;
        private int _savePresetTime;
        private float _characterScale = 1;

        private List<string> _pluginList = new List<string>();
        private string _currentPlugin;
        private MVRPluginManager _manager;
        private Button _removeButton;
        private bool _stopPlugin = false;
        private string _currentMode;

        public bool waitingForUIOpened { get; private set; }

        private readonly string[] _female = new string[]
        {
            "Candy",
            "Evey",
            "Janie",
            "Kayla",
            "Lexi",
            "Maria",
            "Mia",
            "Simone",
            "Tara",
            "Tina"
        };

        private readonly string[] _rigidbodyNames = new string[]
        {
            "hip",
            "pelvis",

            "rThigh",
            "rShin",
            "rFoot",
            "rToe",

            "lThigh",
            "lShin",
            "lFoot",
            "lToe",

            "abdomen",
            "abdomen2",

            "chest",

            "lPectoral",
            "rPectoral",

            "rCollar",
            "rShldr",
            "rForeArm",
            "rHand",

            "lCollar",
            "lShldr",
            "lForeArm",
            "lHand",

            "neck",
            "head"
        };

        private readonly string[] _presets = new string[]
        {
            "Clothing",
            "Hair",
            "Pose",
            "Skin",
            "Appearance"
        };

        public override void Init()
        {
            try
            {
                _targetAtom = containingAtom;
                _manager = _targetAtom.GetComponentInChildren<MVRPluginManager>();
                _currentMode = SuperController.singleton.gameMode.ToString();
                SuperController.singleton.gameMode = SuperController.GameMode.Edit;
                SuperController.singleton.SelectController(_targetAtom.mainController);
                SuperController.singleton.ShowMainHUDAuto();

                if (_currentMode == "Play")
                {
                    SuperController.singleton.gameMode = SuperController.GameMode.Play;
                    SuperController.singleton.HideMainHUD();
                }

                if (_targetAtom.type != "Person")
                {
                    SuperController.LogError("Please attach this plugin to Person");
                    _stopPlugin = true;
                    return;
                }

                if (!FileManagerSecure.DirectoryExists(_pluginDataPath))
                {
                    FileManagerSecure.CreateDirectory(_pluginDataPath);
                }

                var loadPresetButton = CreateButton("Load Presets");
                loadPresetButton.button.onClick.AddListener(LoadPreset);
                JSONStorableAction LoadPresetAct = new JSONStorableAction("Load Presets", LoadPreset);

                var changeManuButton = CreateButton("Change Menu", true);
                changeManuButton.button.onClick.AddListener(ChangeMenu);
                changeManuButton.buttonColor = Color.yellow;
                JSONStorableAction ChangeMenuAct = new JSONStorableAction("Change Menu", ChangeMenu);
                _isTopMenu = true;

                _enableAutoLoad = new JSONStorableBool("Enable Auto Load", false);
                CreateToggle(_enableAutoLoad, true);

                _enableRnd = new JSONStorableBool("Enable Random Load", true);
                var enableRnd = CreateToggle(_enableRnd);
                enableRnd.toggle.onValueChanged.AddListener(delegate { ResetCount(); });

                _enablePositionAdjust = new JSONStorableBool("Enable Position Adjustment", true);
                CreateToggle(_enablePositionAdjust, false);

                _enableSubfolder = new JSONStorableBool("Include Subfolder", true);
                var enableSubfolderToggle = CreateToggle(_enableSubfolder);
                enableSubfolderToggle.toggle.onValueChanged.AddListener(delegate { RefreshList(); });

                List<string> waitCount = new List<string>();
                waitCount.Add("5");
                waitCount.Add("15");
                waitCount.Add("30");
                waitCount.Add("60");
                waitCount.Add("120");
                waitCount.Add("180");

                _selectCount = new JSONStorableStringChooser("Interval", waitCount, "30", "Interval [sec]");
                var sc = CreatePopup(_selectCount, true);

                var sp1 = CreateSpacer(true);
                sp1.height = 21f;

                JSONStorableAction SetClothingPresetPathAct = new JSONStorableAction("Set " + _clothing + " Presets Path", SetClothingPathAct);
                JSONStorableAction SetHairPresetPathAct = new JSONStorableAction("Set " + _hair + " Presets Path", SetHairPathAct);
                JSONStorableAction SetPosePresetPathAct = new JSONStorableAction("Set " + _pose + " Presets Path", SetPosePathAct);
                JSONStorableAction SetSkinPresetPathAct = new JSONStorableAction("Set " + _skin + " Presets Path", SetSkinPathAct);
                JSONStorableAction SetAppearancePresetPathAct = new JSONStorableAction("Set " + _appearance + " Presets Path", SetAppearancePathAct);
                _setPath[_clothing] = SetClothingPath;
                _setPath[_hair] = SetHairPath;
                _setPath[_pose] = SetPosePath;
                _setPath[_skin] = SetSkinPath;
                _setPath[_appearance] = SetAppearancePath;
                _savedPathDic[_clothing] = new JSONStorableString("Clothing Presets Path", "");
                _savedPathDic[_hair] = new JSONStorableString("Hair Presets Path", "");
                _savedPathDic[_pose] = new JSONStorableString("Pose Preset Path", "");
                _savedPathDic[_skin] = new JSONStorableString("Skin Preset Path", "");
                _savedPathDic[_appearance] = new JSONStorableString("Appearance Preset Path", "");

                foreach (var item in _presets)
                {
                    _presetSystemPath[item] = _atomPersonPath + item;
                    _favPathDic[item] = "";
                    _favListDic[item] = new List<string>();
                    _randomOrder[item] = new List<int>();
                    _listDic[item] = new List<string>();
                    _previousNumber[item] = 0;
                    _presetNumber[item] = new JSONStorableString(item + " Current Number", "0");

                    _enablePresetDic[item] = new JSONStorableBool("Enable " + item + " Presets", false);
                    _onlyFavDic[item] = new JSONStorableBool("Favorite " + item + " Only", false);
                    _current[item] = new JSONStorableString("", "");
                    _filesInFolder[item] = new JSONStorableString("", "");
                    MakeUi(item, _setPath[item]);
                }

                _isKeepClothing = new JSONStorableBool("Keep Appearance Clothing", false);
                _isKeepHair = new JSONStorableBool("Keep Appearance Hair", false);
                _isEnableDisappear = new JSONStorableBool("Enable Disappear when moving", true);
                _isKeepScale = new JSONStorableBool("Keep Appearance Scale", false);
                _isNude = new JSONStorableBool("No Clothing", false);

                var presetName = _plugins;
                _enablePresetDic[presetName] = new JSONStorableBool("Enable " + presetName + " Presets", false);
                _enablePresetToggle[presetName] = CreateToggle(_enablePresetDic[presetName]);
                RemoveToggle(_enablePresetToggle[presetName]);

                presetName = _appearance;
                RemoveButton(_setPresetPathButton[presetName]);
                RemoveToggle(_enablePresetToggle[presetName]);
                RemoveToggle(_onlyFavToggle[presetName]);
                RemoveTextField(_currentText[presetName]);
                RemoveTextField(_pathText[presetName]);

                _femaleList = new List<string>(_female);
                _selectTime = int.Parse(_selectCount.val);

                JSONStorableAction SetCurrentPluginAct = new JSONStorableAction("Set Current Plugin Preset", SetCurrentPlugin);
                JSONStorableAction SaveCurrentPresetsAct = new JSONStorableAction("Save Current Presets", SaveCurrentPresets);

                RegisterAction(LoadPresetAct);
                RegisterBool(_enableRnd);
                RegisterBool(_enablePositionAdjust);
                RegisterBool(_enableSubfolder);
                RegisterBool(_enableAutoLoad);
                RegisterStringChooser(_selectCount);

                RegisterAction(SetClothingPresetPathAct);
                RegisterAction(SetHairPresetPathAct);
                RegisterAction(SetPosePresetPathAct);
                RegisterAction(SetSkinPresetPathAct);
                RegisterAction(SetAppearancePresetPathAct);

                RegisterAction(SetCurrentPluginAct);
                RegisterAction(SaveCurrentPresetsAct);

                RegisterBool(_enablePresetDic[_clothing]);
                RegisterBool(_enablePresetDic[_hair]);
                RegisterBool(_enablePresetDic[_pose]);
                RegisterBool(_enablePresetDic[_skin]);
                RegisterBool(_enablePresetDic[_appearance]);
                RegisterBool(_enablePresetDic[_plugins]);

                RegisterBool(_onlyFavDic[_clothing]);
                RegisterBool(_onlyFavDic[_hair]);
                RegisterBool(_onlyFavDic[_pose]);
                RegisterBool(_onlyFavDic[_skin]);
                RegisterBool(_onlyFavDic[_appearance]);

                RegisterString(_savedPathDic[_clothing]);
                RegisterString(_savedPathDic[_hair]);
                RegisterString(_savedPathDic[_pose]);
                RegisterString(_savedPathDic[_skin]);
                RegisterString(_savedPathDic[_appearance]);

                RegisterString(_presetNumber[_clothing]);
                RegisterString(_presetNumber[_hair]);
                RegisterString(_presetNumber[_pose]);
                RegisterString(_presetNumber[_skin]);
                RegisterString(_presetNumber[_appearance]);

                RegisterBool(_isKeepClothing);
                RegisterBool(_isKeepHair);
                RegisterBool(_isEnableDisappear);
                RegisterBool(_isKeepScale);
                RegisterBool(_isNude);
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private void ChangeMenu()
        {
            foreach (var item in _presets)
            {
                RemoveItems(item);
            }

            if (_loadSettingButton != null)
            {
                RemoveButton(_loadSettingButton);
            }

            if (_saveSettingButton != null)
            {
                RemoveButton(_saveSettingButton);
            }

            if (_enablePresetToggle[_plugins] != null)
            {
                RemoveToggle(_enablePresetToggle[_plugins]);
            }

            if (_setPluginPresetButton != null)
            {
                RemoveButton(_setPluginPresetButton);
            }

            if (_spLeft != null)
            {
                RemoveSpacer(_spLeft);
            }

            if (_spRight != null)
            {
                RemoveSpacer(_spRight);
            }

            if (_spLeftBottom != null)
            {
                RemoveSpacer(_spLeftBottom);
            }

            if (_spRightBottom != null)
            {
                RemoveSpacer(_spRightBottom);
            }

            if (_keepClothingToggle != null)
            {
                RemoveToggle(_keepClothingToggle);
            }

            if (_keepHairToggle != null)
            {
                RemoveToggle(_keepHairToggle);
            }

            if (_enableDisappearToggle != null)
            {
                RemoveToggle(_enableDisappearToggle);
            }

            if (_saveCurrentPresetsButton != null)
            {
                RemoveButton(_saveCurrentPresetsButton);
            }

            if (_keepScaleToggle != null)
            {
                RemoveToggle(_keepScaleToggle);
            }

            if (_nudeToggle != null)
            {
                RemoveToggle(_nudeToggle);
            }

            if (_isTopMenu)
            {
                _loadSettingButton = CreateButton("Load Setting", false);
                _loadSettingButton.buttonColor = Color.cyan;
                _saveSettingButton = CreateButton("Save Setting", true);
                _saveSettingButton.buttonColor = Color.cyan;
                _spLeft = CreateSpacer();
                _spLeft.height = 10;
                _spRight = CreateSpacer(true);
                _spRight.height = 10;

                _isTopMenu = false;
                _presetName = _appearance;
                MakeUi(_presetName, SetAppearancePath);

                _loadSettingButton.button.onClick.AddListener(() =>
                {
                    _loopStart = false;
                    if (!FileManagerSecure.DirectoryExists(_pluginDataPath))
                    {
                        FileManagerSecure.CreateDirectory(_pluginDataPath);
                    }
                    var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_pluginDataPath, false, false, true, true);
                    SuperController.singleton.GetMediaPathDialog(LoadSetting, "json", _pluginDataPath, true, true, false, null, false, shortcuts);
                });

                _saveSettingButton.button.onClick.AddListener(() =>
                {
                    _loopStart = false;
                    if (!FileManagerSecure.DirectoryExists(_pluginDataPath))
                    {
                        FileManagerSecure.CreateDirectory(_pluginDataPath);
                    }
                    var fileBrowserUI = SuperController.singleton.fileBrowserUI;
                    fileBrowserUI.SetTitle("Save PresetLoader Setting");
                    fileBrowserUI.fileRemovePrefix = null;
                    fileBrowserUI.hideExtension = false;
                    fileBrowserUI.keepOpen = false;
                    fileBrowserUI.fileFormat = "json";
                    fileBrowserUI.defaultPath = _pluginDataPath;
                    fileBrowserUI.showDirs = true;
                    fileBrowserUI.shortCuts = null;
                    fileBrowserUI.browseVarFilesAsDirectories = true;
                    fileBrowserUI.SetTextEntry(true);
                    fileBrowserUI.Show(SaveSetting);
                    fileBrowserUI.fileEntryField.text = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds + ".json";
                    fileBrowserUI.ActivateFileNameField();
                    fileBrowserUI.showInstallFolderInDirectoryList = true;
                });

                _keepClothingToggle = CreateToggle(_isKeepClothing);
                _keepClothingToggle.toggle.onValueChanged.AddListener(delegate { MakeClothingPreset(); });
                _keepHairToggle = CreateToggle(_isKeepHair, true);
                _keepHairToggle.toggle.onValueChanged.AddListener(delegate { MakeHairPreset(); });
                _keepScaleToggle = CreateToggle(_isKeepScale);
                _keepScaleToggle.toggle.onValueChanged.AddListener(delegate { KeepScale(); });
                _nudeToggle = CreateToggle(_isNude, true);

                _spLeftBottom = CreateSpacer();
                _spLeftBottom.height = 10f;
                _spRightBottom = CreateSpacer(true);
                _spRightBottom.height = 10f;

                _enablePresetToggle[_plugins] = CreateToggle(_enablePresetDic[_plugins]);
                _setPluginPresetButton = CreateButton("Set Current Plugin Preset", true);
                _setPluginPresetButton.buttonColor = Color.green;
                _setPluginPresetButton.button.onClick.AddListener(() =>
                {
                    _loopStart = false;
                    SetCurrentPlugin();
                });
                _enableDisappearToggle = CreateToggle(_isEnableDisappear);
                _saveCurrentPresetsButton = CreateButton("Save Current Presets", true);
                _saveCurrentPresetsButton.buttonColor = new Color(1.0f, 0.7f, 0.7f);
                _saveCurrentPresetsButton.button.onClick.AddListener(SaveCurrentPresets);
            }
            else
            {
                _isTopMenu = true;
                _presetName = _clothing;
                MakeUi(_presetName, SetClothingPath);

                _presetName = _hair;
                MakeUi(_presetName, SetHairPath);

                _presetName = _pose;
                MakeUi(_presetName, SetPosePath);

                _presetName = _skin;
                MakeUi(_presetName, SetSkinPath);
            }
        }

        private void RemoveItems(string item)
        {
            if (_setPresetPathButton[item] != null)
            {
                RemoveButton(_setPresetPathButton[item]);
            }

            if (_enablePresetToggle[item] != null)
            {
                RemoveToggle(_enablePresetToggle[item]);
            }

            if (_onlyFavToggle[item] != null)
            {
                RemoveToggle(_onlyFavToggle[item]);
            }

            if (_currentText[item] != null)
            {
                RemoveTextField(_currentText[item]);
            }

            if (_pathText[item] != null)
            {
                RemoveTextField(_pathText[item]);
            }
        }

        public void MakeUi(string presetName, UnityAction action)
        {
            _setPresetPathButton[presetName] = CreateButton("Set " + presetName + " Presets Path");
            _setPresetPathButton[presetName].button.onClick.AddListener(action);

            _enablePresetToggle[presetName] = CreateToggle(_enablePresetDic[presetName]);
            _onlyFavToggle[presetName] = CreateToggle(_onlyFavDic[presetName], true);
            _onlyFavToggle[presetName].toggle.onValueChanged.AddListener(delegate { ChangePath(presetName); });
            _currentText[presetName] = CreateTextField(_current[presetName]);
            _currentText[presetName].height = _currentPathHeight;
            _pathText[presetName] = CreateTextField(_filesInFolder[presetName], true);
            _pathText[presetName].height = _presetPathHeight;
        }

        public void MakeClothingPreset()
        {
            if (_isKeepClothing.val)
            {
                _targetAtom.GetStorableByID("ClothingPresets").GetStringJSONParam("presetName").val = _clothingPresetsTempFile;
                _targetAtom.GetStorableByID("ClothingPresets").CallAction("StorePreset");
            }
        }

        public void MakeHairPreset()
        {
            if (_isKeepHair.val)
            {
                _targetAtom.GetStorableByID("HairPresets").GetStringJSONParam("presetName").val = _hairPresetsTempFile;
                _targetAtom.GetStorableByID("HairPresets").CallAction("StorePreset");
            }
        }

        public void KeepScale()
        {
            if (_isKeepScale.val)
            {
                _characterScale = _targetAtom.GetStorableByID("rescaleObject").GetFloatJSONParam("scale").val;
            }
        }

        public void LoadSetting(string path)
        {
            if (path != null && path != "")
            {
                var loadData = SuperController.singleton.ReadFileIntoString(path);
                JSONNode jSONNode = JSON.Parse(loadData);
                _enableRnd.val = bool.Parse(jSONNode["Enable Random Load"]);
                _enablePositionAdjust.val = bool.Parse(jSONNode["Enable Position Adjustment"]);
                _enableSubfolder.val = bool.Parse(jSONNode["Include Subfolder"]);
                _enableAutoLoad.val = bool.Parse(jSONNode["Enable Auto Load"]);
                _selectCount.val = jSONNode["Interval"];

                foreach (var item in _presets)
                {
                    _onlyFavDic[item].val = bool.Parse(jSONNode[item][1]);
                    _savedPathDic[item].val = jSONNode[item][2];
                    _presetPath = _presetSystemPath[item];
                    var tempPath = MakeFilename(_savedPathDic[item].val);
                    LoadPresetsList(tempPath);
                    _enablePresetDic[item].val = bool.Parse(jSONNode[item][0]);
                    if (_savedPathDic[item].val == "")
                    {
                        _favPathDic[item] = "";
                        _favListDic[item] = new List<string>();
                        _presetPathDic[item] = "";
                        _presetListDic[item] = new List<string>();
                        _enablePresetDic[item].val = false;
                    }
                    ChangePath(item);
                }
                var presetName = _plugins;
                _enablePresetDic[presetName].val = bool.Parse(jSONNode[presetName][0]);
            }
        }

        public void SaveSetting(string path)
        {
            if (path != null && path != "")
            {
                JSONClass saveData = new JSONClass();
                saveData["Enable Random Load"] = _enableRnd.val.ToString();
                saveData["Enable Position Adjustment"] = _enablePositionAdjust.val.ToString();
                saveData["Include Subfolder"] = _enableSubfolder.val.ToString();
                saveData["Enable Auto Load"] = _enableAutoLoad.val.ToString();
                saveData["Interval"] = _selectCount.val;

                foreach (var item in _presets)
                {
                    saveData[item]["Enable Preset"] = _enablePresetDic[item].val.ToString();
                    saveData[item]["Favorite Only"] = _onlyFavDic[item].val.ToString();
                    saveData[item]["Preset Path"] = _savedPathDic[item].val.ToString();
                }
                var presetName = _plugins;
                saveData[presetName]["Enable Preset"] = _enablePresetDic[presetName].val.ToString();

                if (path.Length < 5 || path.Substring(path.Length - 5) != ".json")
                {
                    path = path + ".json";
                }
                SaveJSON(saveData, path);
            }
        }

        public void ChangePath(string presetName)
        {
            if (_onlyFavDic[presetName].val)
            {
                _filesInFolder[presetName].val = _favPathDic[presetName];
                _listDic[presetName] = new List<string>(_favListDic[presetName]);
            }
            else
            {
                _filesInFolder[presetName].val = _presetPathDic[presetName];
                _listDic[presetName] = new List<string>(_presetListDic[presetName]);
            }
            _previousNumber[presetName] = 0;
            _presetNumber[presetName].val = _previousNumber[presetName].ToString();
            _randomOrder[presetName] = MakeRandomOrder(_listDic[presetName].Count, 0);
        }

        public void ResetCount()
        {
            if (_enableRnd.val)
            {
                foreach (var presetName in _presets)
                {
                    _previousNumber[presetName] = 0;
                    _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                    _randomOrder[presetName] = MakeRandomOrder(_listDic[presetName].Count, 0);
                }
            }
            else
            {
                foreach (var presetName in _presets)
                {
                    _previousNumber[presetName] = 0;
                    _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                }
            }
        }

        public void LoadPreset()
        {
            if (_pluginList == null || _pluginList.Count == 0)
            {
                SetCurrentPlugin();
            }

            if (_enableAutoLoad.val)
            {
                if (_loopStart)
                {
                    _waitCount = _selectTime;
                }
                else
                {
                    _loopStart = true;
                    return;
                }
            }
            else
            {
                _loopStart = false;
                _waitCount = 0;
            }

            ClearPlugins();

            int number;
            var presetName = _appearance;
            if (_enablePresetDic[presetName].val)
            {
                if (_listDic[presetName].Count != 0)
                {
                    if (_listDic[presetName].Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_previousNumber[presetName] >= _listDic[presetName].Count)
                        {
                            _previousNumber[presetName] = 0;
                        }
                        if (_enableRnd.val)
                        {
                            number = _randomOrder[presetName][_previousNumber[presetName]];
                            _previousNumber[presetName] = _previousNumber[presetName] + 1;
                            _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _randomOrder[presetName] = MakeRandomOrder(_listDic[presetName].Count, number);
                                _previousNumber[presetName] = 0;
                                _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            }
                        }
                        else
                        {
                            number = _previousNumber[presetName];
                            _previousNumber[presetName] = number + 1;
                            _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _previousNumber[presetName] = 0;
                                _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            }
                        }
                    }

                    var allGeometry = _targetAtom.GetStorableByID("geometry").GetBoolParamNames();
                    foreach (var item in allGeometry)
                    {
                        if (item.Contains("clothing:"))
                        {
                            if (_targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val)
                            {
                                _targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val = false;
                            }
                        }
                    }
                    _current[presetName].val = _listDic[presetName][number];
                    _targetAtom.GetStorableByID("AppearancePresets").GetStringJSONParam("presetName").val = _listDic[presetName][number];
                    var setStartTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                    var currentTime = setStartTime;
                    do
                    {
                        currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                        if (setStartTime > currentTime)
                        {
                            currentTime += 60000;
                        }
                    }
                    while (currentTime - setStartTime < 200);
                    _targetAtom.GetStorableByID("AppearancePresets").CallAction("LoadPreset");
                }
                else
                {
                    _current[presetName].val = "";
                }
            }
            else
            {
                _current[presetName].val = "";
            }

            if (_isNude.val)
            {
                var allGeometry = _targetAtom.GetStorableByID("geometry").GetBoolParamNames();
                foreach (var item in allGeometry)
                {
                    if (item.Contains("clothing:"))
                    {
                        if (_targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val)
                        {
                            _targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val = false;
                        }
                    }
                }
            }

            if (_isKeepScale.val)
            {
                _targetAtom.GetStorableByID("rescaleObject").GetFloatJSONParam("scale").val = _characterScale;
            }
            _poseWaitStart = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
            _nextPreset = _hair;
        }

        public void ChangeHair()
        {
            int number;
            var presetName = _hair;
            if (_enablePresetDic[presetName].val)
            {
                if (_listDic[presetName].Count != 0)
                {
                    if (_listDic[presetName].Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_previousNumber[presetName] >= _listDic[presetName].Count)
                        {
                            _previousNumber[presetName] = 0;
                        }
                        if (_enableRnd.val)
                        {
                            number = _randomOrder[presetName][_previousNumber[presetName]];
                            _previousNumber[presetName] = _previousNumber[presetName] + 1;
                            _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _randomOrder[presetName] = MakeRandomOrder(_listDic[presetName].Count, number);
                                _previousNumber[presetName] = 0;
                                _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            }
                        }
                        else
                        {
                            number = _previousNumber[presetName];
                            _previousNumber[presetName] = number + 1;
                            _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _previousNumber[presetName] = 0;
                                _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            }
                        }
                    }
                    _current[presetName].val = _listDic[presetName][number];
                    _targetAtom.GetStorableByID("HairPresets").GetStringJSONParam("presetName").val = _listDic[presetName][number];
                    var setStartTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                    var currentTime = setStartTime;
                    do
                    {
                        currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                        if (setStartTime > currentTime)
                        {
                            currentTime += 60000;
                        }
                    }
                    while (currentTime - setStartTime < 200);
                    _targetAtom.GetStorableByID("HairPresets").CallAction("LoadPreset");
                }
                else
                {
                    _current[presetName].val = "";
                }
            }
            else
            {
                _current[presetName].val = "";
            }
            _poseWaitStart = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
        }

        public void ChangeSkin()
        {
            int number;
            var presetName = _skin;
            if (_enablePresetDic[presetName].val)
            {
                if (_listDic[presetName].Count != 0)
                {
                    if (_listDic[presetName].Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_previousNumber[presetName] >= _listDic[presetName].Count)
                        {
                            _previousNumber[presetName] = 0;
                        }
                        if (_enableRnd.val)
                        {
                            number = _randomOrder[presetName][_previousNumber[presetName]];
                            _previousNumber[presetName] = _previousNumber[presetName] + 1;
                            _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _randomOrder[presetName] = MakeRandomOrder(_listDic[presetName].Count, number);
                                _previousNumber[presetName] = 0;
                                _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            }
                        }
                        else
                        {
                            number = _previousNumber[presetName];
                            _previousNumber[presetName] = number + 1;
                            _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _previousNumber[presetName] = 0;
                                _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            }
                        }
                    }

                    var characterName = _targetAtom.GetStorableByID("geometry").GetStringChooserJSONParam("characterSelection").val;
                    if (_femaleList.Contains(characterName) || characterName.Contains("Female"))
                    {
                        _isFemale = true;
                    }
                    else
                    {
                        _isFemale = false;
                    }

                    if (_listDic[presetName][number].Contains(":"))
                    {
                        var skinPresetSplit = _listDic[presetName][number].Split(':');
                        _skinPresetFile = skinPresetSplit[0] + ":/" + _presetSystemPath[presetName] + "/" + skinPresetSplit[1] + ".vap";
                    }
                    else
                    {
                        _skinPresetFile = _presetSystemPath[presetName] + "/" + _listDic[presetName][number] + ".vap";
                    }

                    int posi = _skinPresetFile.LastIndexOf("/");
                    _skinPresetFile = _skinPresetFile.Insert(posi + 1, "Preset_");
                    _loadData = SuperController.singleton.ReadFileIntoString(_skinPresetFile);
                    _jSONNode = JSON.Parse(_loadData);
                    _asArray = _jSONNode["storables"].AsArray;

                    for (int i = 0; i < _asArray.Count; i++)
                    {
                        _asObject = _asArray[i].AsObject;
                        _id = _asObject["id"];
                        if (_id == "geometry" && _asObject["character"] != null)
                        {
                            characterName = _asObject["character"];
                        }
                    }

                    bool _isFemaleInVap = _femaleList.Contains(characterName) || characterName.Contains("Female");
                    if (_isFemale == _isFemaleInVap)
                    {
                        _current[presetName].val = _listDic[presetName][number];
                        _targetAtom.GetStorableByID("SkinPresets").GetStringJSONParam("presetName").val = _listDic[presetName][number];
                        var setStartTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                        var currentTime = setStartTime;
                        do
                        {
                            currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                            if (setStartTime > currentTime)
                            {
                                currentTime += 60000;
                            }
                        }
                        while (currentTime - setStartTime < 200);
                        _targetAtom.GetStorableByID("SkinPresets").CallAction("LoadPreset");
                    }
                    else
                    {
                        _current[presetName].val = "";
                    }
                }
                else
                {
                    _current[presetName].val = "";
                }
            }
            else
            {
                _current[presetName].val = "";
            }
            _poseWaitStart = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
        }

        public void ChangeClothing()
        {
            int number;
            var presetName = _clothing;
            if (_enablePresetDic[presetName].val)
            {
                if (_listDic[presetName].Count != 0)
                {
                    if (_listDic[presetName].Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_previousNumber[presetName] >= _listDic[presetName].Count)
                        {
                            _previousNumber[presetName] = 0;
                        }
                        if (_enableRnd.val)
                        {
                            number = _randomOrder[presetName][_previousNumber[presetName]];
                            _previousNumber[presetName] = _previousNumber[presetName] + 1;
                            _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _randomOrder[presetName] = MakeRandomOrder(_listDic[presetName].Count, number);
                                _previousNumber[presetName] = 0;
                                _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            }
                        }
                        else
                        {
                            number = _previousNumber[presetName];
                            _previousNumber[presetName] = number + 1;
                            _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _previousNumber[presetName] = 0;
                                _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            }
                        }
                    }

                    var allGeometry = _targetAtom.GetStorableByID("geometry").GetBoolParamNames();
                    foreach (var item in allGeometry)
                    {
                        if (item.Contains("clothing:"))
                        {
                            if (_targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val)
                            {
                                _targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val = false;
                            }
                        }
                    }
                    _current[presetName].val = _listDic[presetName][number];
                    _targetAtom.GetStorableByID("ClothingPresets").GetStringJSONParam("presetName").val = _listDic[presetName][number];

                    var setStartTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                    var currentTime = setStartTime;
                    do
                    {
                        currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                        if (setStartTime > currentTime)
                        {
                            currentTime += 60000;
                        }
                    }
                    while (currentTime - setStartTime < 200);
                    _targetAtom.GetStorableByID("ClothingPresets").CallAction("LoadPreset");
                }
                else
                {
                    _current[presetName].val = "";
                }
            }
            else
            {
                _current[presetName].val = "";
            }
            _poseWaitStart = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
        }

        public void ChangePose()
        {
            int number;
            var presetName = _pose;
            if (_enablePresetDic[presetName].val)
            {
                if (_listDic[presetName].Count != 0)
                {
                    if (_listDic[presetName].Count == 1)
                    {
                        number = 0;
                    }
                    else
                    {
                        if (_previousNumber[presetName] >= _listDic[presetName].Count)
                        {
                            _previousNumber[presetName] = 0;
                        }
                        if (_enableRnd.val)
                        {
                            number = _randomOrder[presetName][_previousNumber[presetName]];
                            _previousNumber[presetName] = _previousNumber[presetName] + 1;
                            _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _randomOrder[presetName] = MakeRandomOrder(_listDic[presetName].Count, number);
                                _previousNumber[presetName] = 0;
                                _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            }
                        }
                        else
                        {
                            number = _previousNumber[presetName];
                            _previousNumber[presetName] = number + 1;
                            _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            if (_previousNumber[presetName] >= _listDic[presetName].Count)
                            {
                                _previousNumber[presetName] = 0;
                                _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                            }
                        }
                    }

                    if (_listDic[presetName][number].Contains(":"))
                    {
                        var posePresetSplit = _listDic[presetName][number].Split(':');
                        _posePresetFile = posePresetSplit[0] + ":/" + _presetSystemPath[presetName] + "/" + posePresetSplit[1] + ".vap";
                    }
                    else
                    {
                        _posePresetFile = _presetSystemPath[presetName] + "/" + _listDic[presetName][number] + ".vap";
                    }

                    int iPosi = _posePresetFile.LastIndexOf("/");
                    _posePresetFile = _posePresetFile.Insert(iPosi + 1, "Preset_");
                    _loadData = SuperController.singleton.ReadFileIntoString(_posePresetFile);
                    _jSONNode = JSON.Parse(_loadData);
                    _asArray = _jSONNode["storables"].AsArray;

                    _isPosition = false;
                    _isHip = false;
                    _control = Vector3.zero;

                    _rigidbodyPosiLocal = new Dictionary<string, Vector3>();
                    _rigidbodyRotaLocal = new Dictionary<string, Quaternion>();
                    _rigidbodyPosiWorld = new Dictionary<string, Vector3>();
                    _rigidbodyRotaWorld = new Dictionary<string, Quaternion>();

                    for (int i = 0; i < _asArray.Count; i++)
                    {
                        _asObject = _asArray[i].AsObject;
                        _id = _asObject["id"];
                        if (_id == "control" && _asObject["position"] != null)
                        {
                            _isPosition = true;
                            _posi.x = float.Parse(_asObject["position"][0]);
                            _posi.y = float.Parse(_asObject["position"][1]);
                            _posi.z = float.Parse(_asObject["position"][2]);
                            _control = _posi;
                        }
                        else if (_id == "hipControl" && _asObject["localPosition"] != null)
                        {
                            _posi.x = float.Parse(_asObject["localPosition"][0]);
                            _posi.y = float.Parse(_asObject["localPosition"][1]);
                            _posi.z = float.Parse(_asObject["localPosition"][2]);
                            _hipControl = _posi;
                        }
                        else if (_id == "hip" && _asObject["rootPosition"] != null)
                        {
                            _posi.x = float.Parse(_asObject["rootPosition"][0]);
                            _posi.y = float.Parse(_asObject["rootPosition"][1]);
                            _posi.z = float.Parse(_asObject["rootPosition"][2]);
                            _rota.x = float.Parse(_asObject["rootRotation"][0]);
                            _rota.y = float.Parse(_asObject["rootRotation"][1]);
                            _rota.z = float.Parse(_asObject["rootRotation"][2]);
                            _rigidbodyPosiLocal[_id] = _posi;
                            _rigidbodyRotaLocal[_id] = Quaternion.Euler(_rota);
                            _rigidbodyPosiWorld[_id] = _posi;
                            _rigidbodyRotaWorld[_id] = Quaternion.Euler(_rota);
                            _rootPosi = _posi;
                            _rootRota = _rota;
                            _isHip = true;
                        }
                        else if (Array.IndexOf(_rigidbodyNames, _id) >= 0 && _asObject["position"] != null)
                        {
                            _posi.x = float.Parse(_asObject["position"][0]);
                            _posi.y = float.Parse(_asObject["position"][1]);
                            _posi.z = float.Parse(_asObject["position"][2]);
                            _rota.x = float.Parse(_asObject["rotation"][0]);
                            _rota.y = float.Parse(_asObject["rotation"][1]);
                            _rota.z = float.Parse(_asObject["rotation"][2]);
                            _rigidbodyPosiLocal[_id] = _posi;
                            _rigidbodyRotaLocal[_id] = Quaternion.Euler(_rota);

                            GameObject parent = _targetAtom.GetStorableByID(_id).transform.parent.gameObject;

                            GameObject parentRigid = new GameObject();
                            GameObject currentRigid = new GameObject();

                            parentRigid.transform.position = _rigidbodyPosiWorld[parent.name];
                            parentRigid.transform.rotation = _rigidbodyRotaWorld[parent.name];
                            currentRigid.transform.position = parentRigid.transform.TransformPoint(_posi);
                            currentRigid.transform.rotation = _rigidbodyRotaWorld[parent.name] * _rigidbodyRotaLocal[_id];
                            _rigidbodyPosiWorld[_id] = currentRigid.transform.position;
                            _rigidbodyRotaWorld[_id] = currentRigid.transform.rotation;
                        }
                    }

                    if (_isHip)
                    {
                        float minY = 2f;
                        string minName = "hip";
                        foreach (var item in _rigidbodyNames)
                        {
                            if (_rigidbodyPosiWorld[item] != null)
                            {
                                float thisY = _rigidbodyPosiWorld[item].y;
                                if (minY > thisY)
                                {
                                    minY = thisY;
                                    minName = item;
                                }
                            }
                        }
                        _min = -_rootPosi.y + minY + _hipControl.y;
                    }
                    else
                    {
                        _min = 1f;
                    }

                    _personPosi = _targetAtom.GetStorableByID("control").transform.position;
                    if ((_min < 0 && _enablePositionAdjust.val) || _isPosition)
                    {
                        if (_isEnableDisappear.val)
                        {
                            _targetAtom.tempDisableRender = true;
                        }
                        _activeClothingList = new List<string>();
                        var allGeometry = _targetAtom.GetStorableByID("geometry").GetBoolParamNames();
                        foreach (var item in allGeometry)
                        {
                            if (item.Contains("clothing:"))
                            {
                                if (_targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val)
                                {
                                    _targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val = false;
                                    _activeClothingList.Add(item);
                                }
                            }
                        }
                    }

                    _targetAtom.GetStorableByID("PosePresets").GetStringJSONParam("presetName").val = _listDic[presetName][number];

                    var setStartTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                    var currentTime = setStartTime;
                    do
                    {
                        currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                        if (setStartTime > currentTime)
                        {
                            currentTime += 60000;
                        }
                    }
                    while (currentTime - setStartTime < 200);
                    _targetAtom.GetStorableByID("PosePresets").CallAction("LoadPreset");

                    _loadedPosi = _targetAtom.GetStorableByID("control").transform.position;
                    _current[presetName].val = _listDic[presetName][number];


                    if ((_min < 0 && _enablePositionAdjust.val) || _isPosition)
                    {
                        if (_min >= 0)
                        {
                            _min = 0;
                        }
                        else
                        {
                            if (_enablePositionAdjust.val)
                            {
                                _min -= 0.1f;
                            }
                            else
                            {
                                _min = 0;
                            }
                        }

                        _targetAtom.GetStorableByID("control").transform.position = _personPosi - new Vector3(0, _min, 0);
                    }

                }
                else
                {
                    _current[presetName].val = "";
                }
            }
            else
            {
                _current[presetName].val = "";
            }
            _poseWaitStart = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
        }

        public void MoveRoot()
        {
            if ((_min < 0 && _enablePositionAdjust.val) || _isPosition)
            {
                foreach (var item in _activeClothingList)
                {
                    _targetAtom.GetStorableByID("geometry").GetBoolJSONParam(item).val = true;
                }

                _targetAtom.GetStorableByID("control").GetBoolJSONParam("detachControl").val = true;

                _targetAtom.GetStorableByID("control").transform.position = _personPosi;
                _targetAtom.GetStorableByID("control").transform.rotation = Quaternion.Euler(new Vector3(0, 0, 0));
                _targetAtom.GetStorableByID("control").GetBoolJSONParam("detachControl").val = false;
            }
            _poseWaitStart = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
        }

        public void SetCurrentPlugin()
        {
            _pluginList = new List<string>();
            foreach (Transform transform in _manager.pluginListPanel)
            {
                var urlTransform = transform.Find("Panel").Find("URL").GetComponent<Text>().text;
                var uidTransform = transform.Find("UID").GetComponent<Text>().text;
                var initPlugin = uidTransform + "_" + urlTransform;
                _pluginList.Add(initPlugin.ToString());
            }
        }

        public void ClearPlugins()
        {
            if (_enablePresetDic[_plugins].val && (_enablePresetDic[_clothing].val || _enablePresetDic[_appearance].val))
            {
                TryFindRemoveButton();
                foreach (Transform transform in _manager.pluginListPanel)
                {
                    var urlTransform = transform.Find("Panel").Find("URL").GetComponent<Text>().text;
                    var uidTransform = transform.Find("UID").GetComponent<Text>().text;
                    _currentPlugin = uidTransform + "_" + urlTransform;

                    if (!_pluginList.Contains(_currentPlugin))
                    {
                        RemovePlugin();
                    }
                }
            }
            _poseWaitStart = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
            _nextPreset = _plugins;
        }

        private void RemovePlugin()
        {
            try
            {
                _removeButton = FindRemoveButton();
                _removeButton.onClick.Invoke();
            }
            catch (Exception e)
            {
                SuperController.LogError($"Error removeing plugin : {e}");
            }
        }

        public void TryFindRemoveButton()
        {
            try
            {
                _removeButton = FindRemoveButton();
                if (_removeButton != null && waitingForUIOpened)
                {
                    waitingForUIOpened = false;
                }
            }
            catch (Exception e)
            {
                SuperController.LogError($"Error trying to find remove button for {_currentPlugin} : {e}");
            }
        }

        private Button FindRemoveButton()
        {
            if (_manager.pluginListPanel == null)
            {
                if (!waitingForUIOpened)
                {
                    waitingForUIOpened = true;
                }
                return null;
            }

            foreach (Transform transform in _manager.pluginListPanel)
            {
                var urlTransform = transform.Find("Panel").Find("URL").GetComponent<Text>().text;
                var uidTransform = transform.Find("UID").GetComponent<Text>().text;
                var currentPlugin = uidTransform + "_" + urlTransform;

                if (currentPlugin == _currentPlugin)
                {
                    var buttonTransform = transform.Find("RemoveButton");
                    return buttonTransform.GetComponent<Button>();
                }
            }
            return null;
        }

        public void LoadPlugins()
        {
            if (_enablePresetDic[_plugins].val)
            {
                _appPreset = "";
                if (_enablePresetDic[_appearance].val)
                {
                    if (_targetAtom.GetStorableByID("AppearancePresets").GetStringJSONParam("presetName") != null)
                    {
                        _appPreset = _targetAtom.GetStorableByID("AppearancePresets").GetStringJSONParam("presetName").val;
                        var preset = _appPreset;
                        if (preset != "")
                        {
                            string pluginPath;
                            if (preset.Contains(":"))
                            {
                                string[] varSplit = preset.Split(':');
                                ArrayList allFiles = new ArrayList();
                                SearchFile(_addonPackagesPath, varSplit[0] + ".var", ref allFiles);
                                string findFile = allFiles[0].ToString().Replace("\\", "/");
                                if (varSplit[1].Replace("\\", "/").Contains("/"))
                                {
                                    string[] fileSplit = varSplit[1].Replace("\\", "/").Split('/');
                                    fileSplit[fileSplit.Length - 1] = "Preset_" + fileSplit[fileSplit.Length - 1] + ".vap";
                                    pluginPath = findFile + ":/" + _pluginsPresetPath + "/";
                                    for (int i = 0; i < fileSplit.Length - 1; i++)
                                    {
                                        pluginPath += fileSplit[i] + "/";
                                    }
                                    pluginPath += fileSplit[fileSplit.Length - 1];
                                }
                                else
                                {
                                    pluginPath = findFile + ":/" + _pluginsPresetPath + "/Preset_" + varSplit[1] + ".vap";
                                }
                            }
                            else
                            {
                                if (preset.Replace("\\", "/").Contains("/"))
                                {
                                    string[] fileSplit = preset.Replace("\\", "/").Split('/');
                                    fileSplit[fileSplit.Length - 1] = "Preset_" + fileSplit[fileSplit.Length - 1] + ".vap";
                                    pluginPath = _pluginsPresetPath + "/";
                                    for (int i = 0; i < fileSplit.Length - 1; i++)
                                    {
                                        pluginPath += fileSplit[i] + "/";
                                    }
                                    pluginPath += fileSplit[fileSplit.Length - 1];
                                }
                                else
                                {
                                    pluginPath = _pluginsPresetPath + "/Preset_" + preset + ".vap";
                                }
                            }
                            if (FileManagerSecure.FileExists(pluginPath))
                            {
                                _targetAtom.GetStorableByID("PluginPresets").GetStringJSONParam("presetName").val = preset;
                                var setStartTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                                var currentTime = setStartTime;
                                do
                                {
                                    currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                                    if (setStartTime > currentTime)
                                    {
                                        currentTime += 60000;
                                    }
                                }
                                while (currentTime - setStartTime < 200);
                                _targetAtom.GetStorableByID("PluginPresets").CallAction("MergeLoadPreset");
                            }
                        }
                    }
                }

                if (_enablePresetDic[_clothing].val)
                {
                    if (_targetAtom.GetStorableByID("ClothingPresets").GetStringJSONParam("presetName") != null)
                    {
                        var preset = _targetAtom.GetStorableByID("ClothingPresets").GetStringJSONParam("presetName").val;
                        if (preset != _appPreset)
                        {
                            if (preset != "")
                            {
                                string pluginPath;
                                if (preset.Contains(":"))
                                {
                                    string[] varSplit = preset.Split(':');
                                    ArrayList allFiles = new ArrayList();
                                    SearchFile(_addonPackagesPath, varSplit[0] + ".var", ref allFiles);
                                    string findFile = allFiles[0].ToString().Replace("\\", "/");
                                    if (varSplit[1].Replace("\\", "/").Contains("/"))
                                    {
                                        string[] fileSplit = varSplit[1].Replace("\\", "/").Split('/');
                                        fileSplit[fileSplit.Length - 1] = "Preset_" + fileSplit[fileSplit.Length - 1] + ".vap";
                                        pluginPath = findFile + ":/" + _pluginsPresetPath + "/";
                                        for (int i = 0; i < fileSplit.Length - 1; i++)
                                        {
                                            pluginPath += fileSplit[i] + "/";
                                        }
                                        pluginPath += fileSplit[fileSplit.Length - 1];
                                    }
                                    else
                                    {
                                        pluginPath = findFile + ":/" + _pluginsPresetPath + "/Preset_" + varSplit[1] + ".vap";
                                    }
                                }
                                else
                                {
                                    if (preset.Replace("\\", "/").Contains("/"))
                                    {
                                        string[] fileSplit = preset.Replace("\\", "/").Split('/');
                                        fileSplit[fileSplit.Length - 1] = "Preset_" + fileSplit[fileSplit.Length - 1] + ".vap";
                                        pluginPath = _pluginsPresetPath + "/";
                                        for (int i = 0; i < fileSplit.Length - 1; i++)
                                        {
                                            pluginPath += fileSplit[i] + "/";
                                        }
                                        pluginPath += fileSplit[fileSplit.Length - 1];
                                    }
                                    else
                                    {
                                        pluginPath = _pluginsPresetPath + "/Preset_" + preset + ".vap";
                                    }
                                }

                                if (FileManagerSecure.FileExists(pluginPath))
                                {
                                    _targetAtom.GetStorableByID("PluginPresets").GetStringJSONParam("presetName").val = preset;
                                    var setStartTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                                    var currentTime = setStartTime;
                                    do
                                    {
                                        currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                                        if (setStartTime > currentTime)
                                        {
                                            currentTime += 60000;
                                        }
                                    }
                                    while (currentTime - setStartTime < 200);
                                    _targetAtom.GetStorableByID("PluginPresets").CallAction("MergeLoadPreset");
                                }
                            }
                        }
                    }
                }
            }
            _poseWaitStart = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
            _nextPreset = "Start";
        }

        public List<int> MakeRandomOrder(int number, int lastOne)
        {
            if (number != 0)
            {
                List<int> randomOrder = new List<int>();
                List<int> tempList = new List<int>();
                if (number == 1)
                {
                    randomOrder.Add(0);
                }
                else if (number == 2)
                {
                    if (lastOne == 0)
                    {
                        randomOrder.Add(1);
                        randomOrder.Add(0);
                    }
                    else
                    {
                        randomOrder.Add(0);
                        randomOrder.Add(1);
                    }
                }
                else
                {
                    for (int i = 0; i < number; i++)
                    {
                        tempList.Add(i);
                    }

                    int randomNumber;
                    do
                    {
                        randomNumber = UnityEngine.Random.Range(0, number);
                    }
                    while (randomNumber == lastOne);
                    randomOrder.Add(tempList[randomNumber]);
                    tempList.Remove(randomNumber);
                    number--;
                    do
                    {
                        randomNumber = UnityEngine.Random.Range(0, number);
                        randomOrder.Add(tempList[randomNumber]);
                        tempList.RemoveAt(randomNumber);
                        number--;
                    }
                    while (number > 0);
                }
                return randomOrder;
            }
            else
            {
                return null;
            }
        }

        public void SetClothingPath()
        {
            var presetName = _clothing;
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_presetSystemPath[presetName]);
            _presetPath = _presetSystemPath[presetName];
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetClothingPathAct()
        {
            var presetName = _clothing;
            _presetPath = _presetSystemPath[presetName];
            var path = MakeFilename(_savedPathDic[presetName].val);
            LoadPresetsList(path);
            if (_savedPathDic[presetName].val == "")
            {
                _favPathDic[presetName] = "";
                _favListDic[presetName] = new List<string>();
                _presetPathDic[presetName] = "";
                _presetListDic[presetName] = new List<string>();
                _enablePresetDic[presetName].val = false;
            }
            ChangePath(presetName);
        }

        public void SetHairPath()
        {
            var presetName = _hair;
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_presetSystemPath[presetName]);
            _presetPath = _presetSystemPath[presetName];
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetHairPathAct()
        {
            var presetName = _hair;
            _presetPath = _presetSystemPath[presetName];
            var path = MakeFilename(_savedPathDic[presetName].val);
            LoadPresetsList(path);
            if (_savedPathDic[presetName].val == "")
            {
                _favPathDic[presetName] = "";
                _favListDic[presetName] = new List<string>();
                _presetPathDic[presetName] = "";
                _presetListDic[presetName] = new List<string>();
                _enablePresetDic[presetName].val = false;
            }
            ChangePath(presetName);
        }

        public void SetPosePath()
        {
            var presetName = _pose;
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_presetSystemPath[presetName]);
            _presetPath = _presetSystemPath[presetName];
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetPosePathAct()
        {
            var presetName = _pose;
            _presetPath = _presetSystemPath[presetName];
            var path = MakeFilename(_savedPathDic[presetName].val);
            LoadPresetsList(path);
            if (_savedPathDic[presetName].val == "")
            {
                _favPathDic[presetName] = "";
                _favListDic[presetName] = new List<string>();
                _presetPathDic[presetName] = "";
                _presetListDic[presetName] = new List<string>();
                _enablePresetDic[presetName].val = false;
            }
            ChangePath(presetName);
        }

        public void SetSkinPath()
        {
            var presetName = _skin;
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_presetSystemPath[presetName]);
            _presetPath = _presetSystemPath[presetName];
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetSkinPathAct()
        {
            var presetName = _skin;
            _presetPath = _presetSystemPath[presetName];
            var path = MakeFilename(_savedPathDic[presetName].val);
            LoadPresetsList(path);
            if (_savedPathDic[presetName].val == "")
            {
                _favPathDic[presetName] = "";
                _favListDic[presetName] = new List<string>();
                _presetPathDic[presetName] = "";
                _presetListDic[presetName] = new List<string>();
                _enablePresetDic[presetName].val = false;
            }
            ChangePath(presetName);
        }

        public void SetAppearancePath()
        {
            var presetName = _appearance;
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_presetSystemPath[presetName]);
            _presetPath = _presetSystemPath[presetName];
            SuperController.singleton.GetDirectoryPathDialog(LoadPresetsList, _presetPath, shortcuts, false);
        }

        public void SetAppearancePathAct()
        {
            var presetName = _appearance;
            _presetPath = _presetSystemPath[presetName];
            var path = MakeFilename(_savedPathDic[presetName].val);
            LoadPresetsList(path);
            if (_savedPathDic[presetName].val == "")
            {
                _favPathDic[presetName] = "";
                _favListDic[presetName] = new List<string>();
                _presetPathDic[presetName] = "";
                _presetListDic[presetName] = new List<string>();
                _enablePresetDic[presetName].val = false;
            }
            ChangePath(presetName);
        }

        public void SaveCurrentPresets()
        {
            _isSavePresets = true;
            _saveName = ((int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds).ToString();
            _presetTypePath = "ClothingPresets";
        }

        public void SavePreset()
        {
            SuperController.singleton.SetLeftSelect();
        }

        public string MakeFilename(string path)
        {
            if (path != null && path != "")
            {
                path = path.Replace("\\", "/");
                if (!path.Contains(":"))
                {
                    if (path.Substring(0, 1) == "/")
                    {
                        path = path.Remove(0, 1);
                    }

                    if (!path.Contains(_presetPath))
                    {
                        path = _presetPath + "/" + path;
                    }
                }
                else
                {
                    if (!path.Contains(".var:"))
                    {
                        path = path.Replace(":", ".var:");
                    }

                    string[] div = { ".var:" };
                    var pathSplit = path.Split(div, StringSplitOptions.None);
                    if (!pathSplit[1].Contains(_presetPath))
                    {
                        pathSplit[1] = (_presetPath + "/" + pathSplit[1]).Replace("//", "/");
                    }

                    if (!pathSplit[0].Contains(_addonPackagesPath))
                    {
                        if (FileManagerSecure.FileExists((_addonPackagesPath + "/" + pathSplit[0]).Replace("//", "/")))
                        {
                            pathSplit[0] = (_addonPackagesPath + "/" + pathSplit[0]).Replace("//", "/") + ".var";
                        }
                        else
                        {
                            var filenameSplit = pathSplit[0].Split('.');
                            ArrayList allFiles = new ArrayList();
                            SearchFile(_addonPackagesPath, filenameSplit[0] + "." + filenameSplit[1] + ".*.var", ref allFiles);

                            var searchedFiles = (string[])allFiles.ToArray(typeof(string));
                            int latest = 0;
                            int number = 0;
                            int counter = 0;
                            foreach (var item in searchedFiles)
                            {
                                var itemSplit = item.Replace("\\", "/").Split('/');
                                var findFileSplit = itemSplit[itemSplit.Length - 1].Split('.');
                                if (latest <= int.Parse(findFileSplit[2]))
                                {
                                    latest = int.Parse(findFileSplit[2]);
                                    number = counter;
                                }
                                counter++;
                            }
                            pathSplit[0] = searchedFiles[number];
                        }
                    }
                    else
                    {
                        if (FileManagerSecure.FileExists(pathSplit[0] + ".var"))
                        {
                            pathSplit[0] = pathSplit[0] + ".var";
                        }
                        else
                        {
                            var filePathSplit = pathSplit[0].Split('/');
                            var filenameSplit = filePathSplit[filePathSplit.Length - 1].Split('.');
                            ArrayList allFiles = new ArrayList();
                            SearchFile(_addonPackagesPath, filenameSplit[0] + "." + filenameSplit[1] + ".*.var", ref allFiles);

                            var searchedFiles = (string[])allFiles.ToArray(typeof(string));
                            int latest = 0;
                            int number = 0;
                            int counter = 0;
                            foreach (var item in searchedFiles)
                            {
                                var itemSplit = item.Replace("\\", "/").Split('/');
                                var findFileSplit = itemSplit[itemSplit.Length - 1].Split('.');
                                if (latest <= int.Parse(findFileSplit[2]))
                                {
                                    latest = int.Parse(findFileSplit[2]);
                                    number = counter;
                                }
                                counter++;
                            }
                            pathSplit[0] = searchedFiles[number];
                        }
                    }
                    path = pathSplit[0] + ":/" + pathSplit[1];
                }
                path = path.Replace("//", "/");
                return path;
            }
            return "";
        }

        public void LoadPresetsList(string path)
        {
            if (path != null && path != "")
            {
                path = path.Replace("\\", "/");
                if (!path.Contains(_presetPath) && path.Contains(":"))
                {
                    var pathSplit = path.Split(':');
                }

                _varFileName = "";
                string favPath;
                if (path.Contains(":"))
                {
                    var pathSplit = path.Split(':');
                    _varFullPath = pathSplit[0];
                    var varNameSplit = _varFullPath.Split('/');
                    var varNumber = varNameSplit.Length;
                    _varFullName = varNameSplit[varNumber - 1];
                    _varFileName = _varFullName.Remove(_varFullName.Length - 4, 4);
                    favPath = _filePrefs + _varFileName + pathSplit[1];
                }
                else
                {
                    favPath = path;
                }

                string[] presetFiles;
                string[] favFiles;
                string presetFileList = "";
                string favFileList = "";

                if (_enableSubfolder.val)
                {
                    ArrayList allFiles = new ArrayList();
                    GetAllFiles(path, "Preset_*.vap", ref allFiles);
                    presetFiles = (string[])allFiles.ToArray(typeof(string));
                    _presetList = new List<string>();
                    foreach (var item in presetFiles)
                    {
                        string tempFileName = item.Replace("\\", "/");
                        if (tempFileName.Contains(":"))
                        {
                            tempFileName = _varFileName + ":" + item.Replace("\\", "/").Split(':')[1].Remove(0, 1);
                            tempFileName = tempFileName.Replace(":/", ":");
                        }

                        tempFileName = tempFileName.Replace(_presetPath + "/", "");
                        tempFileName = tempFileName.Remove(tempFileName.LastIndexOf("Preset_"), 7);
                        tempFileName = tempFileName.Remove(tempFileName.LastIndexOf(".vap"), 4);
                        _presetList.Add(tempFileName);
                        if (presetFileList.Length < _jsonstringLimit)
                        {
                            presetFileList = presetFileList + tempFileName + "\n";
                        }
                    }

                    allFiles = new ArrayList();
                    if (FileManagerSecure.DirectoryExists(favPath))
                    {
                        GetAllFiles(favPath, "Preset_*.vap.fav", ref allFiles);
                        favFiles = (string[])allFiles.ToArray(typeof(string));
                        _favList = new List<string>();
                        foreach (var item in favFiles)
                        {
                            string tempFileName = item.Replace("\\", "/");
                            if (tempFileName.Contains(_filePrefs))
                            {
                                var ri = _varFileName.Length;
                                tempFileName = tempFileName.Replace(_filePrefs, "").Remove(ri, 1).Insert(ri, ":");
                            }

                            tempFileName = tempFileName.Replace(_presetPath + "/", "");
                            tempFileName = tempFileName.Remove(tempFileName.LastIndexOf("Preset_"), 7);
                            tempFileName = tempFileName.Remove(tempFileName.LastIndexOf(".vap.fav"), 8);
                            _favList.Add(tempFileName);
                            if (favFileList.Length < _jsonstringLimit)
                            {
                                favFileList = favFileList + tempFileName + "\n";
                            }
                        }
                    }
                    else
                    {
                        _favList = new List<string>();
                        favFileList = "";
                    }
                }
                else
                {
                    presetFiles = FileManagerSecure.GetFiles(path, "Preset_*.vap");
                    _presetList = new List<string>();
                    foreach (var item in presetFiles)
                    {
                        string tempFileName = item.Replace("\\", "/");
                        if (tempFileName.Contains(":"))
                        {
                            tempFileName = _varFileName + ":" + item.Replace("\\", "/").Split(':')[1].Remove(0, 1);
                            tempFileName = tempFileName.Replace(":/", ":");
                        }

                        tempFileName = tempFileName.Replace(_presetPath + "/", "");
                        tempFileName = tempFileName.Remove(tempFileName.LastIndexOf("Preset_"), 7);
                        tempFileName = tempFileName.Remove(tempFileName.LastIndexOf(".vap"), 4);
                        _presetList.Add(tempFileName);
                        if (presetFileList.Length < _jsonstringLimit)
                        {
                            presetFileList = presetFileList + tempFileName + "\n";
                        }
                    }

                    if (FileManagerSecure.DirectoryExists(favPath))
                    {
                        favFiles = FileManagerSecure.GetFiles(favPath, "Preset_*.vap.fav");
                        _favList = new List<string>();
                        foreach (var item in favFiles)
                        {
                            string tempFileName = item.Replace("\\", "/");
                            if (tempFileName.Contains(_filePrefs))
                            {
                                var ri = _varFileName.Length;
                                tempFileName = tempFileName.Replace(_filePrefs, "").Remove(ri, 1).Insert(ri, ":");
                            }

                            tempFileName = tempFileName.Replace(_presetPath + "/", "");
                            tempFileName = tempFileName.Remove(tempFileName.LastIndexOf("Preset_"), 7);
                            tempFileName = tempFileName.Remove(tempFileName.LastIndexOf(".vap.fav"), 8);
                            _favList.Add(tempFileName);
                            if (favFileList.Length < _jsonstringLimit)
                            {
                                favFileList = favFileList + tempFileName + "\n";
                            }
                        }
                    }
                    else
                    {
                        _favList = new List<string>();
                        favFileList = "";
                    }
                }

                foreach (var item in _presets)
                {
                    if (_presetPath == _presetSystemPath[item])
                    {
                        _presetListDic[item] = new List<string>(_presetList);
                        _presetPathDic[item] = presetFileList;
                        _favListDic[item] = new List<string>(_favList);
                        _favPathDic[item] = favFileList;

                        if (_onlyFavDic[item].val)
                        {
                            _filesInFolder[item].val = _favPathDic[item];
                            _listDic[item] = new List<string>(_favListDic[item]);
                        }
                        else
                        {
                            _filesInFolder[item].val = _presetPathDic[item];
                            _listDic[item] = new List<string>(_presetListDic[item]);
                        }
                        _savedPathDic[item].val = path;

                        if (_isRestart)
                        {
                            if (_enablePresetDic[item].val)
                            {
                                _current[item].val = _targetAtom.GetStorableByID(item + "Presets").GetStringJSONParam("presetName").val;
                            }
                            else
                            {
                                _current[item].val = "";
                            }
                            _isRestart = false;
                        }

                        _enablePresetDic[item].val = true;
                        _randomOrder[item] = MakeRandomOrder(_listDic[item].Count, 0);
                    }
                }
            }
        }

        public static void GetAllFiles(string folder, string filter, ref ArrayList allFiles)
        {
            string[] filses = FileManagerSecure.GetFiles(folder, filter);
            allFiles.AddRange(filses);

            string[] subFolders = FileManagerSecure.GetDirectories(folder);
            foreach (string subF in subFolders)
            {
                GetAllFiles(subF, filter, ref allFiles);
            }
        }

        public static void SearchFile(string folder, string filter, ref ArrayList allFiles)
        {
            string[] filses = FileManagerSecure.GetFiles(folder, filter);
            allFiles.AddRange(filses);

            string[] subFolders = FileManagerSecure.GetDirectories(folder);
            foreach (string subF in subFolders)
            {
                SearchFile(subF, filter, ref allFiles);
            }
        }

        public void RefreshList()
        {
            foreach (var item in _presets)
            {
                if (_savedPathDic[item] != null && _savedPathDic[item].val != "")
                {
                    _presetPath = _presetSystemPath[item];
                    _listDic[item] = new List<string>();
                    var path = MakeFilename(_savedPathDic[item].val);
                    var tempEnable = _enablePresetDic[item].val;
                    LoadPresetsList(path);
                    ChangePath(item);
                    _enablePresetDic[item].val = tempEnable;
                }
            }
        }

        // Start is called once before Update or FixedUpdate is called and after Init()
        void Start()
        {
            try
            {
                foreach (var item in _presets)
                {
                    if (_savedPathDic[item] != null && _savedPathDic[item].val != "")
                    {
                        _presetPath = _presetSystemPath[item];
                        _listDic[item] = new List<string>();
                        var path = MakeFilename(_savedPathDic[item].val);
                        var number = int.Parse(_presetNumber[item].val);
                        var loop = _loopStart;
                        var tempEnable = _enablePresetDic[item].val;
                        LoadPresetsList(path);
                        ChangePath(item);
                        _enablePresetDic[item].val = tempEnable;
                        _previousNumber[item] = number;
                        _loopStart = loop;
                    }
                }
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            try
            {

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // FixedUpdate is called with each physics simulation frame by Unity
        void FixedUpdate()
        {
            try
            {
                if (_stopPlugin)
                {
                    return;
                }

                if (_isSavePresets)
                {
                    var kyPath = "ky1001/PresetLoader/";

                    if (_presetTypePath == "ClothingPresets")
                    {
                        _targetAtom.GetStorableByID(_presetTypePath).GetStringJSONParam("presetName").val = kyPath + _saveName;
                        _targetAtom.GetStorableByID(_presetTypePath).CallAction("StorePresetWithScreenshot");
                        SuperController.singleton.SetLeftSelect();
                        _presetTypePath = "HairPresets";
                        _savePresetTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                    }

                    if (_presetTypePath == "HairPresets")
                    {
                        var currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                        if (currentTime < _savePresetTime)
                        {
                            currentTime += 60000;
                        }

                        if (currentTime - _savePresetTime > 200)
                        {
                            _targetAtom.GetStorableByID(_presetTypePath).GetStringJSONParam("presetName").val = kyPath + _saveName;
                            _targetAtom.GetStorableByID(_presetTypePath).CallAction("StorePresetWithScreenshot");
                            SuperController.singleton.SetLeftSelect();
                            _presetTypePath = "SkinPresets";
                            _savePresetTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                        }
                    }

                    if (_presetTypePath == "SkinPresets")
                    {
                        var currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                        if (currentTime < _savePresetTime)
                        {
                            currentTime += 60000;
                        }

                        if (currentTime - _savePresetTime > 200)
                        {
                            _targetAtom.GetStorableByID(_presetTypePath).GetStringJSONParam("presetName").val = kyPath + _saveName;
                            _targetAtom.GetStorableByID(_presetTypePath).CallAction("StorePresetWithScreenshot");
                            SuperController.singleton.SetLeftSelect();
                            _presetTypePath = "AppearancePresets";
                            _savePresetTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                        }
                    }

                    if (_presetTypePath == "AppearancePresets")
                    {
                        var currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                        if (currentTime < _savePresetTime)
                        {
                            currentTime += 60000;
                        }

                        if (currentTime - _savePresetTime > 200)
                        {
                            _targetAtom.GetStorableByID(_presetTypePath).GetStringJSONParam("presetName").val = kyPath + _saveName;
                            _targetAtom.GetStorableByID(_presetTypePath).CallAction("StorePresetWithScreenshot");
                            SuperController.singleton.SetLeftSelect();
                            _presetTypePath = "";
                            _isSavePresets = false;
                        }
                    }
                }

                var waitTime = 1000;
                if (_nextPreset == _plugins)
                {
                    if (_enablePresetDic[_plugins].val)
                    {
                        var cleared = true;
                        foreach (Transform transform in _manager.pluginListPanel)
                        {
                            var urlTransform = transform.Find("Panel").Find("URL").GetComponent<Text>().text;
                            var uidTransform = transform.Find("UID").GetComponent<Text>().text;
                            _currentPlugin = uidTransform + "_" + urlTransform;

                            if (!_pluginList.Contains(_currentPlugin))
                            {
                                cleared = false;
                            }
                        }

                        if (cleared)
                        {
                            _nextPreset = "Start";
                            LoadPlugins();

                            if (_isEnableDisappear.val && _isPosition)
                            {
                                var setStartTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                                var currentTime = setStartTime;
                                do
                                {
                                    currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                                    if (setStartTime > currentTime)
                                    {
                                        currentTime += 60000;
                                    }
                                }
                                while (currentTime - setStartTime < 200);
                            }

                            _targetAtom.tempDisableRender = false;
                            _targetAtom.GetStorableByID("control").GetBoolJSONParam("detachControl").val = false;

                            _currentTime = DateTime.Now.Minute * 60 + DateTime.Now.Second;
                            if (_startTime > _currentTime)
                            {
                                _currentTime += 3600;
                            }

                            if (_selectTime < _currentTime - _startTime && _loopStart)
                            {
                                SuperController.LogMessage("Please select interval time more than " + (_currentTime - _startTime + 1).ToString());
                                _selectCount.val = "30";
                                _selectTime = int.Parse(_selectCount.val);
                            }
                        }
                    }
                    else
                    {
                        _nextPreset = "Start";

                        if (_isEnableDisappear.val && _isPosition)
                        {
                            var setStartTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                            var currentTime = setStartTime;
                            do
                            {
                                currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                                if (setStartTime > currentTime)
                                {
                                    currentTime += 60000;
                                }
                            }
                            while (currentTime - setStartTime < 200);
                        }

                        _targetAtom.tempDisableRender = false;
                        _targetAtom.GetStorableByID("control").GetBoolJSONParam("detachControl").val = false;

                        _currentTime = DateTime.Now.Minute * 60 + DateTime.Now.Second;
                        if (_startTime > _currentTime)
                        {
                            _currentTime += 3600;
                        }

                        if (_selectTime < _currentTime - _startTime && _loopStart)
                        {
                            SuperController.LogMessage("Please select interval time more than " + (_currentTime - _startTime + 1).ToString());
                            _selectCount.val = "30";
                            _selectTime = int.Parse(_selectCount.val);
                        }
                    }
                }

                if (_nextPreset == "MoveRoot")
                {
                    if (_enablePresetDic[_pose].val)
                    {
                        var thisTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                        if (thisTime - _poseWaitStart > waitTime / 2 || ((_poseWaitStart > thisTime) && (60000 - _poseWaitStart + thisTime > waitTime / 2)))
                        {
                            _nextPreset = _plugins;
                            MoveRoot();
                        }
                    }
                    else
                    {
                        _nextPreset = _plugins;
                    }
                }

                if (_nextPreset == _pose)
                {
                    if (_enablePresetDic[_pose].val)
                    {
                        _nextPreset = "MoveRoot";
                        ChangePose();
                    }
                    else
                    {
                        _nextPreset = "MoveRoot";
                    }
                }

                if (_nextPreset == _clothing)
                {
                    if (_enablePresetDic[_clothing].val)
                    {
                        var thisTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                        if (thisTime - _poseWaitStart > waitTime / 2 || ((_poseWaitStart > thisTime) && (60000 - _poseWaitStart + thisTime > waitTime / 2)))
                        {
                            _nextPreset = _pose;
                            ChangeClothing();
                        }
                    }
                    else
                    {
                        if (_isKeepClothing.val && !_isNude.val)
                        {
                            _targetAtom.GetStorableByID("ClothingPresets").GetStringJSONParam("presetName").val = _clothingPresetsTempFile;
                            var setStartTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                            var currentTime = setStartTime;
                            do
                            {
                                currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                                if (setStartTime > currentTime)
                                {
                                    currentTime += 60000;
                                }
                            }
                            while (currentTime - setStartTime < 200);
                            _targetAtom.GetStorableByID("ClothingPresets").CallAction("LoadPreset");
                        }
                        _nextPreset = _pose;
                    }
                }

                if (_nextPreset == _skin)
                {
                    if (_enablePresetDic[_skin].val)
                    {
                        _nextPreset = _clothing;
                        ChangeSkin();
                    }
                    else
                    {
                        _nextPreset = _clothing;
                    }
                }

                if (_nextPreset == _hair)
                {
                    if (_enablePresetDic[_hair].val)
                    {
                        _nextPreset = _skin;
                        ChangeHair();
                    }
                    else
                    {
                        if (_isKeepHair.val)
                        {
                            _targetAtom.GetStorableByID("HairPresets").GetStringJSONParam("presetName").val = _hairPresetsTempFile;
                            var setStartTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                            var currentTime = setStartTime;
                            do
                            {
                                currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                                if (setStartTime > currentTime)
                                {
                                    currentTime += 60000;
                                }
                            }
                            while (currentTime - setStartTime < 200);
                            _targetAtom.GetStorableByID("HairPresets").CallAction("LoadPreset");
                        }
                        _nextPreset = _skin;
                    }
                }

                if (_previousSelectCount != _selectCount.val)
                {
                    _waitCount = 0;
                }

                foreach (var presetName in _presets)
                {
                    if (!_enablePresetDic[presetName].val)
                    {
                        _previousNumber[presetName] = 0;
                        _presetNumber[presetName].val = _previousNumber[presetName].ToString();
                    }
                }

                _selectTime = int.Parse(_selectCount.val);
                if (_enableAutoLoad.val && _loopStart)
                {
                    if (_waitCount <= 0 && _nextPreset == "Start")
                    {
                        _waitCount = _selectTime;
                        _startTime = DateTime.Now.Minute * 60 + DateTime.Now.Second;
                        _nextPreset = _hair;
                        LoadPreset();
                    }

                    _currentTime = DateTime.Now.Minute * 60 + DateTime.Now.Second;
                    if (_currentTime < _startTime)
                    {
                        _currentTime += 3600;
                    }
                    _waitCount = _selectTime - (_currentTime - _startTime);
                }
                else
                {
                    _loopStart = false;
                    _waitCount = _selectTime;
                }

                _previousSelectCount = _selectCount.val;
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // OnDestroy is where you should put any cleanup
        // if you registered objects to supercontroller or atom, you should unregister them here
        void OnDestroy()
        {

        }
    }
}